import 'dart:collection';

import 'package:to_string_pretty/to_string_pretty.dart';

class LruCache<K, V> {
  final int capacity;
  // ignore: prefer_collection_literals
  final _internalMap = LinkedHashMap<K, V>();

  LruCache({this.capacity = 128});

  bool containsKey(K key) => _internalMap.containsKey(key);

  void expire(K key) => _internalMap.remove(key);

  void expireAll() => _internalMap.clear();

  void put(K key, V value) {
    if (!_internalMap.containsKey(key) && _internalMap.length >= capacity) {
      final usedLongestTimeAgo = _internalMap.keys.first;
      _internalMap.remove(usedLongestTimeAgo);
    }
    _internalMap.remove(key);
    _internalMap[key] = value;
  }

  V? get(K key) {
    final removed = _internalMap.remove(key);
    if (removed != null) {
      _internalMap[key] = removed;
    }
    return removed;
  }

  V? operator [](K key) => get(key);

  void operator []=(K key, V value) => put(key, value);

  @override
  String toString() {
    final strs = [
      for (final entry in _internalMap.entries) '${entry.key} => ${entry.value}',
    ];
    if (_internalMap.length >= capacity) {
      strs[0] = '${strs[0]}  <== will be deleted';
    }
    strs[strs.length - 1] = '${strs[strs.length - 1]}  <== recenetly used';
    return toStringPretty(strs);
  }
}
