import 'package:tuple/tuple.dart';

import '../cache.dart';

class Memoized3<V, A1, A2, A3, A4> {
  final LruCache<Tuple4<A1, A2, A3, A4>, V> _cache;
  final V Function(A1, A2, A3, A4) _body;

  Memoized3(V Function(A1, A2, A3, A4) body, {int capacity = 128})
      : _body = body,
        _cache = LruCache(capacity: capacity);

  V call(A1 a1, A2 a2, A3 a3, A4 a4) {
    final tuple = Tuple4(a1, a2, a3, a4);
    final cachedValue = _cache[tuple];
    return cachedValue ?? (_cache[tuple] = _body(a1, a2, a3, a4));
  }

  void expire(A1 a1, A2 a2, A3 a3, A4 a4) => _cache.expire(Tuple4(a1, a2, a3, a4));

  void expireAll() => _cache.expireAll();
}
