import 'package:tuple/tuple.dart';

import '../cache.dart';

class Memoized3<V, A1, A2, A3, A4, A5> {
  final LruCache<Tuple5<A1, A2, A3, A4, A5>, V> _cache;
  final V Function(A1, A2, A3, A4, A5) _body;

  Memoized3(V Function(A1, A2, A3, A4, A5) body, {int capacity = 128})
      : _body = body,
        _cache = LruCache(capacity: capacity);

  V call(A1 a1, A2 a2, A3 a3, A4 a4, A5 a5) {
    final tuple = Tuple5(a1, a2, a3, a4, a5);
    final cachedValue = _cache[tuple];
    return cachedValue ?? (_cache[tuple] = _body(a1, a2, a3, a4, a5));
  }

  void expire(A1 a1, A2 a2, A3 a3, A4 a4, A5 a5) => _cache.expire(Tuple5(a1, a2, a3, a4, a5));

  void expireAll() => _cache.expireAll();
}
